/*
-------------------------------------------------------------------Title: Randomness Calculator Macro---------------------------------------------------------------
Version 1.32
Updated 11 Oct 2018

Description:
- This macro measures the statistical complexity of an image or all the ROIs in it by looking at the size of the GIF file that it is saved as
after subtracting 6 bytes for the fixed header, taking advantage of the fact that GIF files are optimally compressed (Sprott et al., 2002, 
Phy Lett A 297: 267-271).
- The gif file size (with the header size already substracted) will be printed to the Results table.
Notes:
- If your image is already a .gif and you want to analyse the whole image, simply subtract 6 bytes from its file size to get a measure of its
statistical complexity.
- If you're comparing different images/ROIs, the images/ROIs must be of the same dimensions for the comparison to make sense.

Requirements:
- Open the image first, then use this macro
- If multiple areas are to be analysed in one image: one ROI around each area (all identical)
- No other ROIs (e.g. scale bars) in the image

Change Log:
v1.32 (11 Oct 2018): corrected a bug with analysing individual slices.
--------------------------------------------------------------------------------------------------------------------------------------------------------------------
*/
print("\\Clear");
print("-----Randomness Calculator Macro Initiated-----\nGathering options ...");
/* TO DO: Make it batch friendly */

// Dialog to determine what to analyse
items = newArray("Whole image", "ROIs");
sliceNames = newArray(nSlices);
if(nSlices == 1){
	sliceNames[0] = getTitle();
}else{
	for(i=1;i<=nSlices;i++){
		setSlice(i);
		sliceNames[i-1]=getInfo("slice.label");
	}
}
Dialog.create("Analysis options");

Dialog.addChoice("Are you analysing the whole image or the ROIs in it?", items);
if(nSlices>1){
	Dialog.setInsets(5,0,0);
	Dialog.addMessage("----------------------")
	Dialog.setInsets(5,0,0);
	Dialog.addMessage("Multiple slices have been detected in your image. \nPlease check all the slices that you want to analyse:")
	for(i=1;i<=nSlices;i++){
		Dialog.addCheckbox(sliceNames[i-1], true);
	}
}
Dialog.setInsets(5,0,0);
Dialog.addMessage("----------------------")
Dialog.setInsets(5,0,0);
Dialog.addMessage("Do you want to keep the gif file(s) created?:")
Dialog.setInsets(5,0,0);
Dialog.addCheckbox("If checked, a new subfolder will be created with the gif file(s) in it.", true)
Dialog.show();
choice = Dialog.getChoice(); 
numROI = roiManager("count");
if(choice=="ROIs" && numROI==0){
	print("Error: no ROIs detected in the ROI Manager.\n-----Macro Terminated-----");
	selectWindow("Log");
	exit("Error: no ROIs detected. Add ROIs and restart this macro.");
}
if(nSlices>1){
	sliceChoices = newArray(nSlices);
	for(i=0;i<nSlices;i++){
		sliceChoices[i] = Dialog.getCheckbox();
	}
}else{
	sliceChoices = newArray(1);
	sliceChoices[0] = true;
}
keep = Dialog.getCheckbox();

// Determine which slices need to be analysed
if(nSlices==1){
	numSlices=1;
}else{
	numSlices=0;
	for(i=0; i<lengthOf(sliceChoices); i++){
		if(sliceChoices[i] == true)
			numSlices++;
	}
	if(numSlices==0)
		exit("Error: No slices selected.");
}
slicesToAnalyse = newArray(numSlices);
temp = 0;
for(i=1; i<=lengthOf(sliceChoices); i++){
	if(sliceChoices[i-1] == true){
		slicesToAnalyse[temp] = i;
		temp++;
	}
}
run("Duplicate...", "title=working.jpg duplicate");
run("8-bit");

// Get location of image for saving gif files, reopening files if needed, etc.
if(keep == 0)
	dir = getDirectory("temp")+"Randomness_Gifs"+File.separator;
if(keep == 1){
	if(getDirectory("image") == ""){
		dir = getDirectory("Choose a folder to save your GIF files to...");	
	}else{
		dir = getDirectory("image")+"Randomness_Gifs"+File.separator;
	}
}
File.makeDirectory(dir);

if(lastIndexOf(getTitle(),".") > 0){
	title = substring(getTitle(),0,lastIndexOf(getTitle(),"."));
}else{
	title = getTitle();
}


// Perform analysis
print("Calculating statistical complexity ...");

if(choice=="Whole Image"){	
	run("Clear Results");

	waitForUser("IMPORTANT: Ensure that no ROIs or selections are currently active. If an ROI \nis currently active, click on any part of the image outside the ROI, then click OK.");
	if(numSlices != 1)
		saveSize = newArray(numSlices);
	for(i=0; i<numSlices; i++){
		fileName = title+"_"+sliceNames[slicesToAnalyse[i]-1]+".gif";
		fileName = replace(fileName, ":", "_");
		setSlice(slicesToAnalyse[i]);
		run("Duplicate...", "title=&fileName");
		saveAs("gif", dir+fileName);
		size = File.length(dir+fileName)-6;
		if(numSlices != 1)
			saveSize[i] = size;
		setResult("Randomness", i, size);
		setResult("Label", i, sliceNames[slicesToAnalyse[i]-1]);
		close();
	}
	if(numSlices != 1){
		Array.getStatistics(saveSize, min, max, mean);
		setResult("Randomness", numSlices, mean);
		setResult("Label", numSlices, "Mean (of slices analysed only)");
	}
}

if(choice=="ROIs"){

	//Check that All ROIs are of a suitable type
	// ROI types
	// 2D: rectangle, oval, polygon, freehand
	// 1D: line, angle, point, traced, polyline, freeline
//	numROI = roiManager("count");
	checkList  = newArray(numROI);
	indicator = newArray(numROI);
	Array.fill(indicator, true);
	badTypes = newArray("line", "angle", "point", "polyline", "freeline");
	for(i=0; i<numROI; i++){
		roiManager("select", i);
		for(j=0; j<lengthOf(badTypes); j++){
			test = badTypes[j];
			if(Roi.getType == test)
				indicator[i] = 0;
			if(indicator[i] == 0){
				print("Error: "+getInfo("selection.name")+" is a "+Roi.getType+" ROI, and is unsuitable for analysis.");
				break;
			}
		}
	}
	master = 1;
	for(i=0; i<lengthOf(indicator); i++){
		master = indicator[i]*master;
		if(master == 0){
			print("Please remove these ROIs or change them to an area ROI (i.e. rectangle, oval,\ntraced, polygon or freehand) and restart the macro.\n-----Macro terminated-----");
			exit("Error: unsuitable ROI type detected. See the Log window for details.");
		}
	}

	allROIs = Array.getSequence(numROI);
	roiManager("select", allROIs);

	// Prepare the image for analysis
	if(numROI>1)
		roiManager("OR");
	setBackgroundColor(0, 0, 0);
	run("Clear Outside", "stack");

	// Run analysis and save to results window
	run("Clear Results");
	for(i=0; i<numROI; i++){
		roiManager("select", i);
		roiName = getInfo("selection.name");
		if(numSlices != 1)
			saveSize=newArray(numSlices);
		for(j=0; j<numSlices; j++){
			fileName = title+"_"+roiName+"_"+sliceNames[slicesToAnalyse[j]-1]+".gif";
			fileName = replace(fileName, ":", "_");
			setSlice(slicesToAnalyse[j]);
//			run("Duplicate...", "title=&fileName");
			run("Duplicate...", "title=temp");
			saveAs("Gif", dir+fileName);
			size = File.length(dir+fileName)-6;
			if(numSlices != 1)
				saveSize[j] = size;
			setResult(roiName, j, size);
			setResult("Label", j, sliceNames[slicesToAnalyse[j]-1]);
			close();
		}
		if(numSlices != 1){
			Array.getStatistics(saveSize, min, max, mean);
			setResult(roiName, numSlices, mean);
			setResult("Label", numSlices, "Mean (of slices analysed only)");		
		}
	}
}

selectWindow("working.jpg");
close();

// Delete Gifs folder if checkbox is not checked
if(keep == 0){
	fileList = getFileList(dir);
	for(i=0; i<lengthOf(fileList); i++)
		del = File.delete(dir+fileList[i]);
	del = File.delete(dir);
}

updateResults();
print("Statistical Complexity results printed to Results table.");
if(keep==1)
	print("Gif files saved to: <"+dir+">.");
print("-----End of Macro-----");
selectWindow("Log");
selectWindow("Results");
//exit("Analysis complete. See Results Table.");
exit();
