/*
-------------------------------------------Title: Elliptical Fourier Shape Analysis Macro-------------------------------------------
Version: 1.2
Updated: 22 Nov 2016

Description:
- This macro will perform the Elliptical Fourier Analysis (as detailed in "Image Analysis in the Biological
Sciences", chapter 6, page 24) using the Fourier Analysis macro by Boudier and Tupper on all ROIs in
an image. 

Requirements:
- You need to have drawn ROIs around all areas you want to analyse, e.g. by thresholding (Image>Adjust>
Threshold) and then converting to ROIs (Analyze>Particles).
- There should be no other ROIs in the image.
- You need to have installed the Fourier Analysis macro by Boudier and Tupper, which can be found at:
http://imagejdocu.tudor.lu/doku.php?id=plugin:analysis:fourier_shape_analysis:start
----------------------------------------------------------------------------------------------------------------------------------------------------
*/

print("\\Clear");
print("-----Fourier Shape Analysis Macro Initiated-----\nGathering options ...");

// Get user to select analysis options
Dialog.create("Analysis Options");
Dialog.addNumber("Number of descriptors to use for each ROI:", 20);
Dialog.addMessage("More decriptors means more accuracy, but too many may result\nin unwanted noise. 15 to 30 descriptors are recommended.");
Dialog.addMessage("");
Dialog.setInsets(5,0,0);
Dialog.addMessage("IMPORTANT:");
Dialog.addMessage("Ensure that you have drawn a polygon or freehand ROI around every shape you want to \nanalyse, and that there are no other ROIs (e.g. scale bars).");
Dialog.addMessage("If you have not done the above, please press CANCEL, make changes \nto your image and restart this macro.");
Dialog.show();
nDes = Dialog.getNumber(); 

// Check that there are ROIs and that they are of the right type before proceeding
nROIs = roiManager("count");
if(nROIs == 0){
	print("Error: no ROIs detected. Add ROIs to your image and restart the macro.\n-----Macro Terminated-----");
	exit("Error: No ROIs detected.");
}
// ROI types
// 2D: rectangle, oval, polygon, freehand, traced
// 1D: line, angle, point, polyline, freeline
checkList  = newArray(nROIs);
indicator = newArray(nROIs);
Array.fill(indicator, false);
goodTypes = newArray("polygon", "freehand", "traced");
for(i=0; i<nROIs; i++){
	roiManager("select", i);
	for(j=0; j<lengthOf(goodTypes); j++){
		test = goodTypes[j];
		if(Roi.getType == test)
			indicator[i] = 1;
		}
	if(indicator[i] == 0)
		print("Error: "+getInfo("selection.name")+" is a "+Roi.getType+" ROI, and is unsuitable for analysis.");
}
master = 1;
for(i=0; i<lengthOf(indicator); i++){
	master = indicator[i]*master;
	if(master == 0){
		print("Please remove these ROIs or redraw them as a polygon \n or freehand ROI and restart the macro.\n-----Macro terminated-----");
		exit("Error: unsuitable ROI type detected. See the Log window for details.");
	}
}

print("Beginning analysis ...");

// Create an array and get file and directory names to store results
resultsArray = newArray(nROIs*nDes);
desArray = newArray(nDes);
//imageFile = getTitle();
if(lastIndexOf(getTitle(),".") > 0){
	imageFile = substring(getTitle(),0,lastIndexOf(getTitle(),"."));
}else{
	imageFile = getTitle();
}

// Perform the analysis
for(roi=0; roi<nROIs; roi++){
	if((roi+1)%50==0)
		print("Processing ROI "+(roi+1));
	roiManager("Select", roi);
	run("Elliptic Fourier D.", "number="+(nDes+2)+" results"); /*nDes+2 because the first two descriptors are not useful (refer to desription of Elliptical Fourier macro) */
	selectWindow("Results-EFD-"+ imageFile);
	saveAs("results", getDirectory("imagej") + File.separator + "tempfile.xls");
	selectWindow("tempfile.xls");
	run("Close");
	open(getDirectory("imagej") + File.separator + "tempfile.xls");
	for(des=2; des<(nDes+2); des++){
		resultsArray[(roi*nDes)+(des-2)] = getResult("efd", des);
		desArray[des-2] = "Descriptor_"+(des-1);
	}
	selectWindow("Results");
	run("Close");
	File.delete(getDirectory("imagej") + File.separator + "tempfile.xls");
}

// Show results
Array.show("Results", desArray);
roiNames = newArray(roiManager("Count"));
for(i=0; i<nROIs; i++){
	roiManager("select", i);
	roiName = getInfo("selection.name");
	for(j=0; j<nDes; j++){	
//		setResult("ROI_"+(i+1), j, resultsArray[(i*nDes)+j]);
		setResult(roiName, j, resultsArray[(i*nDes)+j]);
	}
}
updateResults();
print("Analysis complete. Results displayed in the Results window.\n-----End of Macro-----");
selectWindow("Results");
exit();

